import { ServiceLogger } from "@effect-aws/commons";
import { Effect, FiberRef, Layer } from "effect";
import { dual } from "effect/Function";
import { globalValue } from "effect/GlobalValue";
/**
 * @since 1.0.0
 * @category codedeploy service config
 */
const currentCodeDeployServiceConfig = globalValue("@effect-aws/client-codedeploy/currentCodeDeployServiceConfig", () => FiberRef.unsafeMake({}));
/**
 * @since 1.0.0
 * @category codedeploy service config
 */
export const withCodeDeployServiceConfig = dual(2, (effect, config) => Effect.locally(effect, currentCodeDeployServiceConfig, config));
/**
 * @since 1.0.0
 * @category codedeploy service config
 */
export const setCodeDeployServiceConfig = (config) => Layer.locallyScoped(currentCodeDeployServiceConfig, config);
/**
 * @since 1.0.0
 * @category adapters
 */
export const toCodeDeployClientConfig = Effect.gen(function* () {
    const { logger: serviceLogger, ...config } = yield* FiberRef.get(currentCodeDeployServiceConfig);
    const logger = serviceLogger === true
        ? yield* ServiceLogger.toClientLogger(ServiceLogger.defaultServiceLogger)
        : (serviceLogger ? yield* ServiceLogger.toClientLogger(ServiceLogger.make(serviceLogger)) : undefined);
    return { logger, ...config };
});
//# sourceMappingURL=CodeDeployServiceConfig.js.map